/*	AMX Mod X script.

	New Year plugin by Lev.

*/

#pragma semicolon 1
#pragma ctrlchar '\'

#include <amxmodx>
#include <amxmisc>
#include <fakemeta>
#include <hamsandwich> 

#define AUTHOR "Lev"
#define PLUGIN "NewYear"
#define VERSION "2009.1"
#define VERSION_CVAR "new_year_version"

// System constants
//#define _DEBUG		// Enable debug output at server console.

#define STEP_DELAY			0.5
#define TASK_SATCHEL_BASE	200
#define TASK_ALLOW_BASE		300

new _maxplayers;
new _hatEnt[33];
new _cachedStringInfoTarget;

new const _satchelModels[][] =
{
	"models/w_gift1.mdl",
	"models/w_gift2.mdl"
};
new const _hatModels[][] =
{
	"models/hats/santa_hat_blue.mdl",
	"models/hats/santa_hat_red.mdl",
	"models/hats/santa_hat_straight.mdl"
};
new const _stepSounds[][] =
{
	"player/pl_snow1_louder.wav",
	"player/pl_snow2_louder.wav",
	"player/pl_snow3_louder.wav",
	"player/pl_snow4_louder.wav"
};
/*
new const _jumpSounds[][] =
{
	"player/pl_snow5.wav",
	"player/pl_snow6.wav"
};
*/


public plugin_init() 
{
	register_plugin(PLUGIN, VERSION, AUTHOR);
	register_cvar(VERSION_CVAR, VERSION, FCVAR_SPONLY | FCVAR_SERVER | FCVAR_UNLOGGED);

	register_event("CurWeapon","CurWeaponEvent","be","1=1");
	register_forward(FM_SetModel, "SetModelEvent", 1);
	//register_forward(FM_PlayerPreThink, "fwd_PlayerPreThink", 0);
	RegisterHam(Ham_Spawn, "player", "HamPlayerSpawnPostEvent", 1);
	RegisterHam(Ham_Weapon_PrimaryAttack, "weapon_satchel", "SatchelAttackEvent", 1);

	_cachedStringInfoTarget = engfunc(EngFunc_AllocString, "info_target");
	_maxplayers = get_maxplayers();

	set_task(0.3, "MapStartTask");
}

new _allow[33];
public SatchelAttackEvent(ent)
{
	new id = pev(ent, pev_owner);
	if (_allow[id] == 1)
		return HAM_IGNORED;

	if (_allow[id] == 2)
	{
		set_task(0.1, "SatchelBlowedTask", TASK_SATCHEL_BASE + id);
		return HAM_IGNORED;
	}

	_allow[id] = 1;
	set_task(0.5, "Allow", TASK_ALLOW_BASE + id);
	return HAM_IGNORED;
}

public Allow(id)
{
	id -= TASK_ALLOW_BASE;
	_allow[id] = 2;
	set_task(0.1, "SatchelBlowedTask", TASK_SATCHEL_BASE + id);
}

public SatchelBlowedTask(id)
{
	id -= TASK_SATCHEL_BASE;
	_allow[id] = 0;
	CurWeaponEvent(id);
}


public plugin_precache()
{
	precache_model("models/v_gift.mdl");
	precache_model("models/p_gift.mdl");
	for (new i = 0; i < sizeof(_satchelModels); i++)
		precache_model(_satchelModels[i]);
	for (new i = 0; i < sizeof(_hatModels); i++)
		precache_model(_hatModels[i]);
	for (new i = 0; i < sizeof(_stepSounds); i++)
		precache_sound(_stepSounds[i]);
}



//****************************************
//*                                      *
//*  Weapon models changing              *
//*                                      *
//****************************************

/// Replace entities models on map on start
public MapStartTask()
{
	static ent, classname[33], model[33];
	ent = engfunc(EngFunc_FindEntityInSphere, _maxplayers, Float:{ 0.0, 0.0, 0.0 }, 4800.0);
	while (ent)
	{
		pev(ent, pev_classname, classname, charsmax(classname));
		if (containi(classname, "satchel") != -1)
		{
			pev(ent, pev_model, model, charsmax(model));
			SetModelEvent(ent, model);
		}
		ent = engfunc(EngFunc_FindEntityInSphere, ent, Float:{ 0.0, 0.0, 0.0 }, 4800.0);
	}
}

/// Replace entities models on map on spawn
public SetModelEvent(ent, const szModel[])
{
	if (!pev_valid(ent))
		return FMRES_IGNORED;
	
	if (equali(szModel, "models/w_satchel.mdl"))
	{
		engfunc(EngFunc_SetModel, ent, _satchelModels[random_num(0, sizeof(_satchelModels) - 1)]);
		return FMRES_SUPERCEDE;
	}
	return FMRES_IGNORED;
}

/// Replace hands entities on weapon change
public CurWeaponEvent(id)
{
	if (!is_user_alive(id))
		return PLUGIN_CONTINUE;
	static model[32];

	pev(id, pev_viewmodel2, model, charsmax(model));
	if (equali(model, "models/v_satchel.mdl"))
		set_pev(id, pev_viewmodel2, "models/v_gift.mdl");

	pev(id, pev_weaponmodel2, model, charsmax(model));
	if (equali(model, "models/p_satchel.mdl"))
		set_pev(id, pev_weaponmodel2, "models/p_gift.mdl");

	return PLUGIN_CONTINUE;
}



//****************************************
//*                                      *
//*  Hat models                          *
//*                                      *
//****************************************

public HamPlayerSpawnPostEvent(id)
{
	if (!pev_valid(_hatEnt[id]))
	{
		new ent = _hatEnt[id] = engfunc(EngFunc_CreateNamedEntity, _cachedStringInfoTarget);
		set_pev(ent, pev_movetype, MOVETYPE_FOLLOW);
		set_pev(ent, pev_aiment, id);
		engfunc(EngFunc_SetModel, ent, _hatModels[random_num(0, sizeof(_hatModels) - 1)]);
	}
	return HAM_IGNORED;
}
